/* menu.c the main menu of the game where the user can select to play a new
 * game, change to fullscreen / window or quit, it really should have a setting
 * option too!
 *
 * Copyright (c) 2019, Chris Camacho (codifies) http://bedroomcoders.co.uk/captcha/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <stdio.h>

#include "raylib.h"

#include "screens.h"
#include "button.h"

static bool menuExit;

static ScreenCallbacks* scb; // structure of callbacks for this screen

#define MAX_BUTTONS 3
static button* buttons[MAX_BUTTONS];

enum buttonNames {NEW_BUTTON = 0, QUIT_BUTTON, FULLSCREEN_BUTTON};

static Texture fullTex;
static Texture winTex;

//------------------------------------------------------------------------------
// update
//------------------------------------------------------------------------------

static void update()
{
    Vector2 m = GetMousePosition();
    // scale and move the mouse position to correspond with render texture position / scale
    m.x = m.x / ScreenScale;
    m.x = m.x +(((((float)gameScreenWidth*ScreenScale)-GetScreenWidth())/ScreenScale)/2.0);
    m.y = m.y / ScreenScale;
    m.y = m.y +(((((float)gameScreenHeight*ScreenScale)-GetScreenHeight())/ScreenScale)/2.0);

    // update all of the buttons
    for (int i=0; i < MAX_BUTTONS; i++) {
        updateButton(buttons[i], m);
    }

    // if the quit button is pressed signal the menu needs to close
    if (buttons[QUIT_BUTTON]->btnAction) {
        menuExit = true;
    }

    // hide the mouse cursor and then switch to the game screen
    if (buttons[NEW_BUTTON]->btnAction) {
        HideCursor();
        currentScreen = scr_GAME;
    }

    // toggle full screen, if you do this without changing the window size
    // in some platforms you get a low res fullscreen, the work around is
    // to change the window size and then later on actually toggle fullscreen
    if (buttons[FULLSCREEN_BUTTON]->btnAction) {
        toggleScreen = 4;   // work around...
        fullScreen = !fullScreen;
        if (fullScreen) {
            SetWindowSize(gameScreenWidth, gameScreenHeight);
            buttons[FULLSCREEN_BUTTON]->texture = winTex;
        } else {
            buttons[FULLSCREEN_BUTTON]->texture = fullTex;
        }
    }
}

//------------------------------------------------------------------------------
// render
//------------------------------------------------------------------------------
static void render(fnt* font)
{
    ClearBackground(WHITE);

    // game title
    BeginShaderMode(font->fontShader);    // Activate SDF font shader
        DrawTextEx(font->font, "Raylib Shooter", (Vector2){630,64}, 60, -1, BLACK);
        DrawTextEx(font->font, "Main Menu", (Vector2){800,160}, 45, -1, BLACK);
    EndShaderMode();            // Activate our default shader for next drawings

    // render all the buttons
    for (int i = 0; i < MAX_BUTTONS; i++) {
        renderButton(buttons[i]);
    }
}

// does this screen want to exit ?
static bool done()
{
    return menuExit;
}

//------------------------------------------------------------------------------
// unload / free everything from this screens setup
//------------------------------------------------------------------------------
static void destroy()
{
    freeButton(buttons[NEW_BUTTON]);
    freeButton(buttons[QUIT_BUTTON]);
    freeButtonTx(buttons[FULLSCREEN_BUTTON]);
    UnloadTexture(fullTex);
    UnloadTexture(winTex);
    free(scb);
}

//------------------------------------------------------------------------------
// setup and allocate everything needed for this screen
//------------------------------------------------------------------------------
ScreenCallbacks* setUpMenu()
{
    // screen callbacks for main...
    scb = malloc(sizeof(ScreenCallbacks));
    scb->update = update;
    scb->render = render;
    scb->destroy = destroy;
    scb->done = done;

    buttons[NEW_BUTTON] = createButton("data/newGameButton.png", gameScreenWidth/2-100, 300);

    winTex = LoadTexture("data/windowedButton.png");
    fullTex = LoadTexture("data/fullscreenButton.png");
    if (!fullScreen) {
        buttons[FULLSCREEN_BUTTON] = createButtonTx(fullTex, gameScreenWidth/2-100, 400);
    } else {
        buttons[FULLSCREEN_BUTTON] = createButtonTx(winTex, gameScreenWidth/2-100, 400);
    }

    buttons[QUIT_BUTTON] = createButton("data/quitButton.png", gameScreenWidth/2-100, 800);

    menuExit = false;

    return scb;
}


